package com.censoft.censoftrongtong.utils;


import cn.hutool.core.collection.CollUtil;
import com.deepoove.poi.XWPFTemplate;
import com.deepoove.poi.data.Rows;
import com.deepoove.poi.plugin.table.LoopRowTableRenderPolicy;
import com.deepoove.poi.policy.TableRenderPolicy;
import com.deepoove.poi.template.ElementTemplate;
import com.deepoove.poi.template.run.RunTemplate;
import com.deepoove.poi.util.TableTools;
import org.apache.poi.xwpf.usermodel.*;

import java.util.Collection;
import java.util.List;
import java.util.stream.Stream;


public class MyLoopRowTableRenderPolicy extends LoopRowTableRenderPolicy {

    /**
     * 创建单元格的数量，因为每个票都不同，创建不同的单元格合并才会合并成一整行
     */
    private Integer cellSize;

    public MyLoopRowTableRenderPolicy() {
        super();
        //默认13
        this.cellSize = 13;
    }

    public MyLoopRowTableRenderPolicy(int cellSize) {
        super();
        if(cellSize <= 1){
            throw new RuntimeException("参数错误，只能是大于1的整数");
        }
        this.cellSize = cellSize;
    }

    @Override
    public void render(ElementTemplate eleTemplate, Object data, XWPFTemplate template) {
        //数据必须是集合类，否则不应该使用这个渲染器
        if (data instanceof Collection && CollUtil.isEmpty((Collection) data)) {
            //为空时默认补充一行无
            renderEmpty(eleTemplate);
        } else {
            super.render(eleTemplate, data, template);
        }
    }

    /**
     * 渲染空数据，在表格钟默认增加一行无
     *
     * @param eleTemplate
     */
    private void renderEmpty(ElementTemplate eleTemplate) {
        RunTemplate runTemplate = (RunTemplate) eleTemplate;
        XWPFRun run = runTemplate.getRun();
        //消除标签值
        run.setText("", 0);
        XWPFTableCell tagCell = (XWPFTableCell) ((XWPFParagraph) run.getParent()).getBody();
        //获取当前表格
        XWPFTable table = tagCell.getTableRow().getTable();
        //获取模板所在行，即表格标签下一行
        int rowIndex = getTemplateRowIndex(tagCell) + 1;
        //删除表格标签下一行
        table.removeRow(rowIndex);
        //插入一行
        XWPFTableRow row = table.insertNewTableRow(rowIndex);
        Stream.iterate(0, i -> i + 1)
                .limit(cellSize).forEach(i -> row.createCell());
        TableTools.mergeCellsHorizonal(table, rowIndex, 0, cellSize - 1);
        try {
            TableRenderPolicy.Helper.renderRow(table.getRow(rowIndex),
                    Rows.of("无").center().textFontFamily("宋体").textFontSize(11).create());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private int getTemplateRowIndex(XWPFTableCell tagCell) {
        XWPFTableRow tagRow = tagCell.getTableRow();
        return this.getRowIndex(tagRow);
    }

    private int getRowIndex(XWPFTableRow row) {
        List<XWPFTableRow> rows = row.getTable().getRows();
        return rows.indexOf(row);
    }

}
